@@ -2,7 +2,7 @@ package Devel::Examine::Subs;
 use warnings; 
 use strict;
 
-our $VERSION = '1.23';
+our $VERSION = '1.24';
 
 use Carp; 
 use Data::Dumper; 
@@ -11,6 +11,7 @@ use Devel::Examine::Subs::Preprocessor;
 use Devel::Examine::Subs::Prefilter; 
 use File::Find; 
 use PPI; 
+use Symbol;
 use Tie::File;
 
 sub new {
@@ -31,6 +32,11 @@ sub run {
     my $self = shift;
     my $p = shift;
 
+    # we're starting the run
+    # gets set to true at end of _core()
+
+    $self->_run_end(0);
+
     $self->_config($p);
 
     # do something different for a dir
@@ -41,12 +47,18 @@ sub run {
     else {
         $self->_core($self->{params});
     }
+
 }
 sub run_directory {
 
     my $self = shift;
     my $p = shift;
 
+    # we're starting the run
+    # gets set to true at end of _core()
+
+    $self->_run_end(0);
+
     $self->_config($p);
 
     my @files;
@@ -81,7 +93,12 @@ sub run_directory {
 
         $struct{$file} = $data if $exists;
     }
-    
+
+    # we've got to clean up core/config here
+
+    $self->_run_end(1);    
+    $self->_clean_core_config();
+
     return \%struct;
 }
 sub _config {
@@ -89,6 +106,49 @@ sub _config {
     my $self = shift;
     my $p = shift;
 
+    my %valid_params = (
+
+        # persistent
+
+        file => 1,
+        extensions => 1,
+        regex => 1,
+        copy => 1,
+        diff => 1,
+        no_indent => 1,
+
+        # persistent - core phases
+
+        pre_proc => 1,
+        pre_filter => 1,
+        engine => 1,
+
+        # transient
+
+        directory => 0,
+        search => 0,
+        replace => 0,
+        injects => 0,
+        code => 0,
+        include => 0,
+        exclude => 0,
+        lines => 0,
+        module => 0,
+        pre_proc_dump => 0,
+        pre_filter_dump => 0,
+        engine_dump => 0,
+        core_dump => 0,
+        pre_proc_return => 0,
+        pre_filter_return => 0,
+        engine_return => 0,
+        config_dump => 0,
+    );
+
+    $self->{valid_params} = \%valid_params;
+
+    # clean config
+
+    $self->_clean_config(\%valid_params, $p);
 
     for my $param (keys %$p){
 
@@ -105,6 +165,45 @@ sub _config {
         print Dumper $self->{params};
     }
 }
+sub _clean_config {
+    my $self = shift;
+    my $config_vars = shift; # href of valid params
+    my $p = shift; # href of params passed in
+
+    for my $var (keys %$config_vars){
+       
+        last if ! $self->_run_end();
+
+        # skip if it's a persistent var
+
+        next if $config_vars->{$var} == 1;
+
+        delete $self->{params}{$var};
+    }
+
+    # delete non-valid params
+
+    for my $param (keys %$p){
+        if (! exists $config_vars->{$param}){
+            delete $p->{$param};
+        }
+    }
+}
+sub _clean_core_config {
+    # deletes core phase info after each run
+
+    my $self = shift;
+
+    my @core_phases = qw( 
+        pre_proc
+        pre_filter
+        engine
+    );
+
+    for (@core_phases){
+        delete $self->{params}{$_};
+    }
+}
 sub _file {
 
     my $self = shift;
@@ -112,6 +211,40 @@ sub _file {
 
     $self->{params}{file} = $p->{file} // $self->{params}{file};
 
+    # if a module was passed in, dig up the file
+
+    if ($self->{params}{file} =~ /::/){
+
+        my $module = $self->{params}{file};
+        (my $file = $module) =~ s|::|/|g;
+        $file .= '.pm';
+       
+        my $module_is_loaded;
+        
+        if (! $INC{$file}){
+            eval { require $module; };
+
+            if ($@){
+                $@ = "Devel::Examine::Subs::_file() speaking ... " .
+                     "Can't transform module to a file name\n\n"
+                     . $@;
+                croak $@;
+            }
+        }
+        else {
+            $module_is_loaded = 1;
+        }
+
+        # set the file param
+
+        $self->{params}{file} = $INC{$file};
+       
+        if (! $module_is_loaded){ 
+            delete_package $module;
+            delete $INC{$file};
+        }
+    }
+
     # configure directory searching for run()
 
     if (-d $self->{params}{file}){
@@ -136,7 +269,7 @@ sub _core {
 
     my $search = $self->{params}{search};
     my $file = $self->{params}{file};
-    
+
     # pre processor
 
     my $data;
@@ -197,7 +330,13 @@ sub _core {
 
         exit;
     }
-    
+
+    # do some config cleaning
+    # run_directory() takes care of cleanup if it's enabled
+
+    $self->_run_end(1) if ! $self->{params}{directory};
+    $self->_clean_core_config() if ! $self->{params}{directory};
+
     $self->{data} = $subs;
     return $subs;
 }
@@ -469,6 +608,14 @@ sub _engine {
 
     return $cref;
 }
+sub _run_end {
+    my $self = shift;
+    my $value = shift;
+
+    $self->{run_end} = $value if defined $value;
+
+    return $self->{run_end};
+}
 sub pre_procs {
 
     my $self = shift;
@@ -672,7 +819,10 @@ sub add_functionality {
 
     push @TIE_file, @code;
 }
-
+sub valid_params {
+    my $self = shift;
+    return %{$self->{valid_params}};
+}
 sub _pod{} #vim placeholder 1; 
 __END__
 
@@ -684,23 +834,30 @@ Devel::Examine::Subs - Get info, search/replace and inject code in Perl file sub
 
     use Devel::Examine::Subs;
 
-    my $file = 'perl.pl'; # or directory name
+    my $file = 'perl.pl'; # or directory, or module name (D::E::S)
     my $search = 'string';
 
     my $des = Devel::Examine::Subs->new({file => $file);
 
+Get all the subs as objects
+
+    $aref = $des->objects(...)
+
+    for my $sub (@$aref){
+        $sub->name();       # name of sub
+        $sub->start();      # number of first line in sub
+        $sub->end();        # number of last line in sub
+        $sub->num_lines();  # number of lines in sub
+        $sub->code();       # entire sub code from file
+        $sub->lines();      # lines that match search term
+
+    }
+
+
 Get all sub names in a file
 
     my $aref = $des->all();
 
-Print all subs within each Perl file under a directory
-
-    my $files = $des->all({ file => 'lib/Devel/Examine' });
-
-    for my $file (keys %$files){
-        print "$file\n";
-        print join('\t', @{$files->{$file}});
-    }
 
 Get all subs containing "string" in the body
 
@@ -730,28 +887,19 @@ Inject code into sub after a search term (preserves previous line's indenting)
         croak 'big bad error';
     }
 
-Get all the subs as objects
-
-    $aref = $des->objects(...)
-
-    for my $sub (@$aref){
-        $sub->name();       # name of sub
-        $sub->start();      # number of first line in sub
-        $sub->end();        # number of last line in sub
-        $sub->num_lines();  # number of lines in sub
-        $sub->code();       # entire sub code from file
-        $sub->lines();      # see next example...
-
-    }
-
 Print out all lines in all subs that contain a search term
 
-    my $lines_with_search_term = $sub->lines();
+    my $subs = $des->objects(...);
 
-    for (@$lines_with_search_term){
-        my ($line_num, $text) = split /:/, $_, 2;
-        say "Line num: $line_num";
-        say "Code: $text\n";
+    for my $sub (@$subs){
+    
+        my $lines_with_search_term = $sub->lines();
+
+        for (@$lines_with_search_term){
+            my ($line_num, $text) = split /:/, $_, 2;
+            say "Line num: $line_num";
+            say "Code: $text\n";
+        }
     }
 
 The structures look a bit differently when 'file' is a directory. You need to add one more layer of extraction.
@@ -764,8 +912,22 @@ The structures look a bit differently when 'file' is a directory. You need to ad
         }
     }
 
+Print all subs within each Perl file under a directory
 
+    my $files = $des->all({ file => 'lib/Devel/Examine' });
 
+    for my $file (keys %$files){
+        print "$file\n";
+        print join('\t', @{$files->{$file}});
+    }
+
+All methods can include or exclude specific subs
+
+    my $has = $des->has({ include => [qw(dump private)] });
+
+    my $missing = $des->missing({ exclude => ['this', 'that'] });
+
+    # note that 'exclude' param renders 'include' invalid
 
 
 =head1 DESCRIPTION
@@ -808,20 +970,26 @@ Gather information about subroutines in Perl files (and in-memory modules), with
 
 =head1 METHODS
 
-=head2 C<new({ file =E<gt> $filename })>
+All parameters are passed in via a single hash reference in all public methods. See the L<PARAMETERS> for the full list of params, and which ones are persistent across runs using the same object.
 
-Instantiates a new object.
 
-Takes the name of a file to search. If $filename is a directory, it will be searched recursively for files. You can set any 
-and all parameters this module uses in any method, however, only the 'file', 'extensions' and 'regex' params are guaranteed to stay persistent, so best to supply your desired params to each call. 
 
-See the L<PARAMETERS> section for optional parameters that can and perhaps should be set here. 
+=head2 C<new()>
+
+Mandatory parameters: C<{ file =E<gt> $filename }>
+
+Instantiates a new object. If C<$filename> is a directory, we'll iterate through it finding all Perl files.
+
+Only specific params are guaranteed to stay persistent throughout a run on the same object, and are best set in C<new()>. These parameters are C<file>, C<extensions>, C<regex>, C<copy>, C<no_indent> and C<diff>. 
+
 
 
 
 
 =head2 C<all()>
 
+Mandatory parameters: None
+
 Returns an array reference containing the names of all subroutines found in the file.
 
 
@@ -829,14 +997,18 @@ Returns an array reference containing the names of all subroutines found in the
 
 
 
-=head2 C<has({ search =E<gt> $text })>
+=head2 C<has()>
 
-Returns an array reference containing the names of the subs where the subroutine contains the text.
+Mandatory parameters: C<{ search =E<gt> 'term' }>
+
+Returns an array reference containing the names of the subs where the subroutine contains the search text.
 
 
 
 
-=head2 C<missing({ search =E<gt> $text })>
+=head2 C<missing()>
+
+Mandatory parameters: 'C<{ search =E<gt> 'term' }>
 
 The exact opposite of has.
 
@@ -846,24 +1018,31 @@ The exact opposite of has.
 
 
 
+=head2 C<module()>
 
-=head2 C<module({ module =E<gt> 'Devel::Examine::Subs' } )>
+Mandatory parameters: { module =E<gt> 'Devel::Examine::Subs' }
 
 Returns an array reference containing the names of all subs found in the module's namespace symbol table.
 
 
 
 
-=head2 C<lines({ search =E<gt> $text })>
+=head2 C<lines()>
 
-Gathers together all line text and line number of all subs where the sub contains lines matching the search term.
+Mandatory parameters: { search =E<gt> $text }
 
-Returns a hash reference with the sub name as the key, the value being an array reference which contains a hash reference in the format line_number =E<gt> line_text.
+Gathers together all line text and line number of all subs where the subroutine contains lines matching the search term.
+
+Returns a hash reference with the subroutine name as the key, the value being an array reference which contains a hash reference in the format line_number =E<gt> line_text.
 
 
 
 
-=head2 C<search_replace({ search =E<gt> 'this', replace =E<gt> 'that', copy =E<gt> 'file.ext' })>
+=head2 C<search_replace()>
+
+Mandatory parameters: { search =E<gt> 'this', replace =E<gt> 'that' }
+
+Core optional parameter: C<copy =E<gt> 'filename.txt'>
 
 Search for lines that contain certain text, and replace the search term with the replace term. If the optional parameter 'copy' is sent in, a copy of the original file will be created in the current directory with the name specified, and that file will be worked on instead. Good for testing to ensure The Right Thing will happen in a production file.
 
@@ -874,7 +1053,9 @@ This method will create a backup copy of the file with the same name appended wi
 
 
 
-=head2 C<inject_after({ search =E<gt> 'this', code =E<gt> \@code })>
+=head2 C<inject_after()>
+
+Mandatory parameters: { search =E<gt> 'this', code =E<gt> \@code }
 
 Injects the code in C<@code> into the sub within the file, where the sub contains the search term. The same indentation level of the 
 line that contains the search term is used for any new code injected. Set C<no_indent> parameter to a true value to disable this 
@@ -896,6 +1077,10 @@ Optional parameters:
 
 See C<search_replace()> for a description of how this parameter is used.
 
+=item C<injects>
+
+How many injections do you want to do per sub? See L<PARAMETERS> for more details.
+
 =back
 
 
@@ -917,6 +1102,11 @@ Returns a list of all available built-in pre engine filter modules.
 Returns a list of all available built-in 'engine' modules.
 
 
+=head2 C<valid_params()>
+
+Returns a hash where the keys are valid parameter names, and the value is a bool where if true, the parameter is persistent (remains between calls on the same object) and if false, the param is transient, and will be made C<undef> after each method call finishes.
+
+
 =head2 C<run()>
 
 All public methods call this method internally. The public methods set certain variables (filters, engines etc). You can get the same 
@@ -933,11 +1123,13 @@ effect programatically by using C<run()>. Here's an example that performs the sa
 This allows for very fine-grained interaction with the application, and makes it easy to write new engines and for testing.
 
 
+
+
+
 =head2 C<add_functionality()>
 
 WARNING!: This method is highly experimental and is used for developing internal processors only. Only 'engine' is functional, and only 
-half way. It's simply a proof-of-concept of the 'Processor' structure which I will be incorporating into a new module template system 
-that allows people to replicate the base structure of this module (less the data and processors). DO NOT USE.
+half way.
 
 While writing new processors, set the processor type to a callback within the local working file. When the code performs the actions you 
 want it to, put a comment line before the code with C<#<des>> and a line following the code with C<#</des>>. DES will slurp in all of 
@@ -984,63 +1176,103 @@ properly.
 
 =head1 PARAMETERS
 
-There are various optional global parameters that can be used. These should be set in C<new()>, unless you want them only briefly in which case just call them within the user public methods.
+There are various parameters that can be used to change the behaviour of the application. Some are persistent across calls, and others aren't.
+ You can change or null any/all parameters in any call, but some should be set in the C<new()> method (set it and forget it).
+
+The following list are persistent parameters, which need to be manually changed or nulled. Consider setting these in C<new()>.
 
 =over 4
 
 =item C<file>
 
-The name of a file, or a directory. If set in C<new>, you can omit it from all subsequent method calls until you want it changed. Once changed in a call, the updated value will remain persistent until changed again.
+State: Persistent
 
+The name of a file, directory or module name in the standard form C<Devel::Examine::Subs>. Will convert module name to a file name if the module is installed on the system. It'll require the module temporarily and then 'un'-require it immediately after use.
+
+If set in C<new>, you can omit it from all subsequent method calls until you want it changed. Once changed in a call, the updated value will remain persistent until changed again.
+
+
+=item C<extensions>
+
+State: Persistent
+
+By default, we load only C<*.pm> and C<*.pl> files. Use this parameter to load different files. Only useful when a directory is passed in as opposed to a file. This parameter is persistent until manually reset and should be set in C<new>.
+
+Values: Array reference where each element is the name of the extension (less the dot). For example, C<[qw(pm pl)]> is the default.
+
+
+=item C<copy>
+
+State: Persistent
+
+For methods that write to files, you can optionally work on a copy that you specify in order to review the changes before modifying a production file.
+
+Set this parameter to the name of an output file. The original file will be copied to this name, and we'll work on this copy.
+
+
+=item C<regex>
+
+State: Persistent
+
+Set to a true value, all values in the 'search' parameter become regexes. For example with regex on, C</thi?s/> will match "this", but without regex, it won't. This parameter is persistent; it remains until reset manually.
+
+
+=item C<no_indent>
+
+State: Persistent
+
+In the processes that write new code to files, the indentation level of the line the search term was found on is used for inserting the 
+new code by default. Set this parameter to a true value to disable this feature and set the new code at the beginning column of the 
+file.
 
 =item C<diff>
 
+State: Persistent
+
 Not yet implemented. 
 
 Compiles a diff after each edit using the methods that edit files.
 
+=back
+
+The following parameters are not persistent, ie. they get reset before entering the next call on the DES object. They must be passed in to each subsequent call if the effect is still desired.
 
 
+=over 4
 
 =item C<include>
 
+State: Transient
+
 An array reference containing the names of subs to include. This (and C<exclude>) tell the Processor phase to generate only these subs, significantly reducing the work that needs to be done in subsequent method calls.
 
 
 
 =item C<exclude>
 
+State: Transient
+
 An array reference of the names of subs to exclude. See C<include> for further details.
 
 Note that C<exclude> renders C<include> useless.
 
 
 
-=item C<no_indent>
-
-In the processes that write new code to files, the indentation level of the line the search term was found on is used for inserting the 
-new code by default. Set this parameter to a true value to disable this feature and set the new code at the beginning column of the 
-file.
 
 =item C<injects>
 
+State: Transient
+
 Informs C<inject_after()> how many injections to perform. For instance, if a search term is found five times in a sub, how many of those do you want to inject the code after?
 
 Default is 1. Set to a higher value to achieve more injects. Set to a negative integer to inject after all.
 
-=item C<regex>
-
-Set to a true value, all values in the 'search' parameter become regexes. For example with regex on, C</thi?s/> will match "this", but without regex, it won't. This parameter is persistent; it remains until reset manually.
-
-=item C<extensions>
-
-By default, we load only C<*.pm> and C<*.pl> files. Use this parameter to load different files. Only useful when a directory is passed in as opposed to a file. This parameter is persistent until manually reset and should be set in C<new>.
-
-Values: Array reference where each element is the name of the extension (less the dot). For example, C<['pm', 'pl']> is the default.
 
 
 =item C<pre_proc_dump>, C<pre_filter_dump>, C<engine_dump>, C<core_dump>
 
+State: Transient
+
 Set to 1 to activate, C<exit()>s after completion.
 
 Print to STDOUT using Data::Dumper the structure of the data following the respective phase. The C<core_dump> will print the state of 
@@ -1059,6 +1291,8 @@ likewise, C<1> will dump after the first.
 
 =item C<pre_proc_return>, C<pre_filter_return>, C<engine_return>
 
+State: Transient
+
 Returns the structure of data immediately after being processed by the respective phase. Useful for writing new 'phases'. (See "SEE 
 ALSO" for details).
 
@@ -1066,14 +1300,11 @@ NOTE: C<pre_filter_return> does not behave like C<pre_filter_dump>. It will only
 
 
 
-=item C<clean_config>
-
-Resets all configuration variables back to C<undef>, less the persistent global ones ('file', 'extensions', 'regex').
-
-
 
 =item C<config_dump>
 
+State: Transient
+
 Prints to STDOUT with Data::Dumper the current state of all loaded configuration parameters.
 
 
